<?xml version='1.0' ?>
<xsl:stylesheet 
  version='1.0'
  xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
  xmlns:redirect="http://xml.apache.org/xalan/redirect"
  extension-element-prefixes="redirect"
  >
<!-- Reference Documents:
For details regarding xsl tags refer to http://www.w3.org/TR/xslt11
For details regarding xpath expressions refer to http://www.w3.org/TR/xpath
 -->
 
 <!-- output file is HTML -->
<xsl:output method='html' />

<!--
    default image directory is current dir
    use -PARAM imagedir <someotherdir> to specify an alternate image directory
-->
<xsl:param name='imagedir' select='"./"'/>

<xsl:variable name='lcase' select="'abcdefghijklmnopqrstuvwxyz'" />
<xsl:variable name='ucase' select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'" />

  <xsl:template name='generateBookmark'>
    <xsl:element name="A">
      <xsl:attribute name="name">
        <xsl:value-of select="generate-id()"/>
      </xsl:attribute>
    </xsl:element>
    <xsl:if test="./@id">
    	<xsl:element name="A">
    		<xsl:attribute name="name">
    			<xsl:value-of select="@id"/>
    		</xsl:attribute>
    	</xsl:element>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name='navHeader'>
  	<xsl:param name='prevFname'/>
  	<xsl:param name='nextFname'/>
  	
    <TABLE WIDTH="100%" BORDER="0" CELLPADDING="0" CELLSPACING="0" CLASS="NAVHEADER">
    <TR>
      <TH COLSPAN="3" ALIGN="center">
        <I CLASS="EMPHASIS">
          <xsl:value-of select='//title[1]'/>
        </I>
      </TH>
    </TR>
    <TR>
      <TD WIDTH="15%" ALIGN="left" VALIGN="bottom">
      	<!-- link to previous document -->
      	<xsl:if test="not($prevFname='.html')">
      	<xsl:element name="A">
      		<xsl:attribute name="HREF">
	      			<xsl:value-of select="$prevFname"/>
      		</xsl:attribute>
      		<xsl:text>&#60;&#60;&#60; Previous</xsl:text>
      	</xsl:element>
      	</xsl:if>
      </TD>

      <TD WIDTH="70%" ALIGN="center" VALIGN="top">
      	<!-- chapter title is not displayed in the main page (sub-document) of a chapter.
      		The chapter title is displayed in header for sub-documents and sub-pages of the chapter -->
	  		<xsl:if test="name()='chapter'"><xsl:value-of select="''"/></xsl:if>
  			<xsl:if test="name()='section'"><xsl:value-of  select="parent::chapter/title"/></xsl:if>
      </TD>

      <TD WIDTH="15%" ALIGN="right" VALIGN="bottom">
      	<!-- link to next document -->
      	<xsl:if test="not($nextFname='.html')">
      	<xsl:element name="A">
      		<xsl:attribute name="HREF">
	      			<xsl:value-of select="$nextFname"/>
      		</xsl:attribute>
					<xsl:text>Next &#62;&#62;&#62;</xsl:text>
      	</xsl:element>
      	</xsl:if>
      </TD>
    </TR>
    </TABLE>
    <HR/>
  </xsl:template>

  <xsl:template name='navFooter'>
  	<xsl:param name='prevFname'/>
  	<xsl:param name='nextFname'/>

  	<HR/>
    <TABLE WIDTH="100%" BORDER="0" CELLPADDING="0" CELLSPACING="0" CLASS="NAVHEADER">
    <TR>
    	<TD>
      	<!-- link to previous document -->
      	<xsl:if test="not($prevFname='.html')">
      	<xsl:element name="A">
      		<xsl:attribute name="HREF">
	      			<xsl:value-of select="$prevFname"/>
      		</xsl:attribute>
      		<xsl:text>&#60;&#60;&#60; Previous</xsl:text>
      	</xsl:element>
      	</xsl:if>
      </TD>

      <TD WIDTH="70%" ALIGN="center" VALIGN="top">
      		<xsl:element name="A">
      			<xsl:attribute name="HREF">index.html</xsl:attribute>
      			Home
      		</xsl:element>
      		<xsl:if test="name()='section'">
      		<br/>
      		<xsl:element name="A">
      			<xsl:attribute name="HREF">
      				<xsl:value-of select="concat(ancestor::chapter[1]/@id, '.html')"/>
      			</xsl:attribute>
      			Up
      		</xsl:element>
	   			</xsl:if>
      </TD>

      <TD WIDTH="15%" ALIGN="right" VALIGN="top">
      	<!-- link to next document -->
      	<xsl:if test="not($nextFname='.html')">
      	<xsl:element name="A">
      		<xsl:attribute name="HREF">
	      			<xsl:value-of select="$nextFname"/>
      		</xsl:attribute>
					<xsl:text>Next &#62;&#62;&#62;</xsl:text>
      	</xsl:element>
      	</xsl:if>
      </TD>
    </TR>
    </TABLE>
  </xsl:template>
  
  <xsl:template name='newPage'>
  	<xsl:variable name='prevFname'>
			  		<xsl:if test="name()='chapter'">
			  			<xsl:if test="count(preceding::chapter[1]/section[title]) &lt; 2">
			  			<xsl:value-of select="preceding::chapter[1]/@id"/>
			  			</xsl:if>
			  			<xsl:if test="count(preceding::chapter[1]/section[title]) > 1">
			  				<xsl:value-of select="generate-id(preceding::chapter[1]/section[title][last()])"/>
			  			</xsl:if>
			  		</xsl:if>
			  		
			  		<xsl:if test="name()='section'">
			  			<xsl:if test="count(preceding-sibling::section[title]) = 1">
			  				<xsl:value-of select="parent::chapter/@id"/>
			  			</xsl:if>
			  			<xsl:if test="count(preceding-sibling::section[title]) > 1">
			  				<xsl:value-of select="generate-id(preceding-sibling::section[title][1])"/>
			  			</xsl:if>
			  		</xsl:if>
			  		<xsl:text>.html</xsl:text>
  	</xsl:variable>
  	
    <xsl:variable name='nextFname'>
    	<xsl:if test="name()='chapter'">
    		<xsl:if test="count(section[title]) > 1">
				<xsl:value-of select="generate-id(section[title][2])"/>
				</xsl:if>
				<xsl:if test="count(section[title]) &lt; 2">
					<xsl:value-of select="following-sibling::chapter[1]/@id"/>
				</xsl:if>
    	</xsl:if>

    	<xsl:if test="name()='section'">
    		<xsl:if test="count(following-sibling::section[title]) > 0">
	    		<xsl:value-of select="generate-id(following-sibling::section[title][1])"/>
    		</xsl:if>
    		<xsl:if test="count(following-sibling::section[title]) = 0">
    			<xsl:value-of select="following::chapter[1]/@id"/>
    		</xsl:if>
    	</xsl:if>
  		<xsl:text>.html</xsl:text>
  	</xsl:variable>
  
    	<html>
      <head>
        <LINK REL="stylesheet" TYPE="text/css" HREF="jonas-tutorial.css"/>
        <title><xsl:value-of select='child::title[1]'/></title>
      </head>
    <body>
      <xsl:comment><xsl:value-of select="name()"/></xsl:comment>
      <xsl:choose>

        <xsl:when test='name()="book" '>
          <div class='BOOK'>
          <div class='TITLEPAGE'>
          <xsl:apply-templates />
          </div>
          </div>
        </xsl:when>

        <xsl:when test='name()="chapter" '>
        	<xsl:call-template name='navHeader'>
        		<xsl:with-param name='prevFname' select='$prevFname'/>
        		<xsl:with-param name='nextFname' select='$nextFname'/>
        	</xsl:call-template>
          <xsl:apply-templates />
        	<xsl:call-template name='navFooter'>
        		<xsl:with-param name='prevFname' select='$prevFname'/>
        		<xsl:with-param name='nextFname' select='$nextFname'/>
        	</xsl:call-template>
        </xsl:when>

        <xsl:when test='name()="section" '>
        	<xsl:call-template name='navHeader'>
        		<xsl:with-param name='prevFname' select='$prevFname'/>
        		<xsl:with-param name='nextFname' select='$nextFname'/>
        	</xsl:call-template>
          <xsl:call-template name='generateBookmark'/>
          <xsl:apply-templates />
        	<xsl:call-template name='navFooter'>
        		<xsl:with-param name='prevFname' select='$prevFname'/>
        		<xsl:with-param name='nextFname' select='$nextFname'/>
        	</xsl:call-template>
        </xsl:when>

        <xsl:when test='name()="legalnotice" '>
          <DIV CLASS='centered-text'>
            <I CLASS='EMPHASIS'><xsl:value-of select='ancestor::bookinfo/title[1]'/></I>
          </DIV>
          <HR/>
          <xsl:apply-templates />
          <HR/>
          <DIV CLASS='centered-text'>
            <A href='index.html'>Home</A>
          </DIV>
        </xsl:when>

        <xsl:otherwise>
          <xsl:apply-templates />
        </xsl:otherwise>

      </xsl:choose>

    </body>
    </html>
  </xsl:template>

  <xsl:template match="/" >
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="chapter">
    <redirect:write select="concat(@id,'.html')">
      <xsl:call-template name='newPage'/>
     </redirect:write>
  </xsl:template>

  <xsl:template match="legalnotice">
    <redirect:write select="concat(generate-id(),'.html')">
      <xsl:call-template name='newPage'/>
    </redirect:write>
  </xsl:template>

  <xsl:template match="title">
  	<xsl:choose>
  		<xsl:when test="name(parent::*) = 'formalpara'">
        <B><xsl:apply-templates/></B><BR/>
  		</xsl:when>
  		
  		<xsl:when test="name(parent::*) = 'figure'">
  		</xsl:when>
  		
  		<xsl:otherwise>
        <H1><xsl:apply-templates/></H1>
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

  <xsl:template match="figure">
  	<xsl:element name="A">
  		<xsl:attribute name="NAME">
  			<xsl:value-of select="@id"/>
  		</xsl:attribute>
  	</xsl:element>
   <xsl:apply-templates />
   <SPAN CLASS="FIGURE">
   	Figure <xsl:value-of select="1 + count(preceding-sibling::figure)"/>: <xsl:value-of select='title'/>
   	</SPAN>
  </xsl:template>
  
  <xsl:template match="blockquote">
  	<DIV CLASS='BLOCKQUOTE'>
  			<!-- select all nodes except the attribution which is displayed as the last row, -->
  			<xsl:apply-templates select="descendant::*[not(name()='attribution')]"/>
				<DIV CLASS='ATTRIBUTION'>
  					<!--attribution follows text of quote and is right aligned using CSS -->
  					--<xsl:value-of select='attribution'/>
				</DIV>
  	</DIV>
  </xsl:template>
  
  <xsl:template match="email">
  <TT CLASS="EMAIL">
  	<xsl:element name="A">
  		<xsl:attribute name="HREF">
  			<xsl:value-of select="concat('mailto:', .)"/>
  		</xsl:attribute>
  	&lt;<xsl:apply-templates/>&gt;
  	</xsl:element>
  </TT>
 
 	</xsl:template>
    
  <xsl:template match="para">
    <p><xsl:apply-templates /></p>
  </xsl:template>
    
  <xsl:template match="superscript">
    <sup><xsl:apply-templates /></sup>
  </xsl:template>
    
  <xsl:template match="section[not(title)] ">
    <p><xsl:apply-templates /></p>
  </xsl:template>

  <xsl:template match="section[title] " >
    <xsl:choose>
      <!-- redirect output to a new html file if the following conditions are true:
      1. this is not the first section with a title
      2. this section is not a child of another section with a title
      -->
      <xsl:when test='count(preceding-sibling::section[title]) > 0 and count(parent::section[title]) = 0'>
        <redirect:write select="concat(generate-id(),'.html')">
        	<xsl:call-template name='newPage' />
        </redirect:write>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name='generateBookmark'/>
        <p><xsl:apply-templates /></p>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="formalpara">
    <p><xsl:apply-templates /></p>
  </xsl:template>

  <xsl:template match="listitem">
    <li><xsl:apply-templates /></li>
  </xsl:template>

  <xsl:template match="itemizedlist">
    <ul><xsl:apply-templates /></ul>
  </xsl:template>

  <xsl:template match='filename'>
    <TT CLASS="FILENAME"><xsl:apply-templates /></TT>
  </xsl:template>

  <xsl:template match='envar'>
    <TT CLASS="ENVAR" ><xsl:apply-templates /></TT>
  </xsl:template>

  <xsl:template match='screen'>
    <TABLE BORDER="0" BGCOLOR="#E0E0E0" WIDTH="90%" >
      <TR>
        <TD>
          <pre class='SCREEN'>
            <xsl:apply-templates />
          </pre>
        </TD>
      </TR>
    </TABLE>
  </xsl:template>

    <xsl:template match='note'>
    	<DIV CLASS="NOTE">
    		<P>
        <TABLE CLASS="NOTE" WIDTH="100%" BORDER="0" >
            <TR>
            <TD WIDTH="25" ALIGN="CENTER" VALIGN="TOP">
                <xsl:element name='img'>
                    <xsl:attribute name='src'>
                        <xsl:value-of select="concat( $imagedir, 'stylesheet-images/note.gif' )"/>
                    </xsl:attribute>
                    <xsl:attribute name='hspace'>5</xsl:attribute>
                    <xsl:attribute name='alt'>Note</xsl:attribute>
                    <xsl:attribute name='width'>24</xsl:attribute>
                    <xsl:attribute name='height'>24</xsl:attribute>
                </xsl:element>
            </TD>
            <TD ALIGN="LEFT" VALIGN="TOP" >
                <xsl:apply-templates />
            </TD>
            </TR>
        </TABLE>
        </P>
        </DIV>
    </xsl:template>

  <xsl:template match='varname'>
    <TT	CLASS="VARNAME"	> <xsl:apply-templates /> </TT>
  </xsl:template>

  <xsl:template match='literal'>
    <TT	CLASS="LITERAL"	> <xsl:apply-templates /> </TT>
  </xsl:template>

  <xsl:template match='quote'>
    <SPAN CLASS="QUOTE">&quot;<xsl:apply-templates/>&quot;</SPAN>
  </xsl:template>

  <xsl:template match='guimenu'>
    <B CLASS="GUIMENU"><xsl:apply-templates/></B>
  </xsl:template>

  <xsl:template match='emphasis'>
    <I CLASS="EMPHASIS"><xsl:apply-templates /></I>
  </xsl:template>

  <xsl:template match='guilabel'>
    <TT CLASS="GUILABEL"><B><xsl:apply-templates/></B></TT>
  </xsl:template>

  <xsl:template match='guibutton'>
    <B CLASS="GUIBUTTON"><xsl:apply-templates/></B>
  </xsl:template>

  <xsl:template match='command'>
    <B CLASS="COMMAND"><xsl:apply-templates/></B>
  </xsl:template>

  <xsl:template match='option'>
    <span CLASS="OPTION"><xsl:apply-templates/></span>
  </xsl:template>

  <xsl:template match='citetitle'>
    <I CLASS="CITETITLE"><xsl:apply-templates/></I>
  </xsl:template>
  
  <xsl:template match="orderedlist">
  	<OL TYPE="1">  		<xsl:apply-templates/>   	</OL>
  </xsl:template>

  <xsl:template match='warning'>
    <DIV CLASS="WARNING">
    <P>
    <TABLE CLASS="WARNING" WIDTH="100%" BORDER="0" >
    <TR>
      <TD WIDTH="25" ALIGN="CENTER" VALIGN="TOP">
            <xsl:element name='img'>
                <xsl:attribute name='src'>
                    <xsl:value-of select="concat( $imagedir, 'stylesheet-images/warning.gif' )"/>
                </xsl:attribute>
                <xsl:attribute name='hspace'>5</xsl:attribute>
                <xsl:attribute name='alt'>Note</xsl:attribute>
                <xsl:attribute name='width'>24</xsl:attribute>
                <xsl:attribute name='height'>24</xsl:attribute>
            </xsl:element>
      </TD>
      <TD ALIGN="LEFT" VALIGN="TOP" ><xsl:apply-templates /></TD>
    </TR>
    </TABLE>
    </P>
    </DIV>
  </xsl:template>

  <xsl:template match='ulink'>
    <xsl:element name='a'>
      <xsl:attribute name='href'>
        <xsl:value-of select='@url'/>
      </xsl:attribute>
      <xsl:attribute name='target'>_top</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match='change'>
    <span style='background-color:beige'>
      <xsl:apply-templates/>
      </span>
  </xsl:template>

  <xsl:template match='book'>
    <xsl:call-template name='newPage'/>
  </xsl:template>

  <xsl:template match='bookinfo'>
    <xsl:apply-templates />
    <hr/>
  </xsl:template>

  <xsl:template match='date'>
    <xsl:comment>
      <xsl:apply-templates />
    </xsl:comment>
  </xsl:template>

  <xsl:template match='authorgroup'>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match='author'>
    <DIV CLASS="AUTHOR"><xsl:apply-templates/></DIV>					
  </xsl:template>

  <xsl:template match='firstname'>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match='surname' >
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match='mediaobject' >
    <DIV CLASS='MEDIAOBJECT'><xsl:apply-templates /></DIV>				
  </xsl:template>

  <xsl:template match='imageobject' >
    <P><xsl:apply-templates /></P>
  </xsl:template>

  <xsl:template match='imagedata' >
    <xsl:element name='IMG'>
      <xsl:attribute name='SRC'>
        <xsl:value-of select='concat( $imagedir, @fileref)'/>
      </xsl:attribute>
    </xsl:element>
  </xsl:template>

  <xsl:template match='affiliation'>
    <DIV CLASS="AFFILIATION"><xsl:apply-templates /></DIV>
  </xsl:template>
  
  <xsl:template match='orgname'>
    <SPAN CLASS='ORGNAME'>
      <xsl:apply-templates />
      <br/>
    </SPAN>
  </xsl:template>

  <xsl:template match='address'>
    <SPAN CLASS='ADDRESS'>
      <xsl:apply-templates />
      <BR/>
    </SPAN>
  </xsl:template>
    
  <xsl:template match='othercredit'>
  </xsl:template>

  <xsl:template match='edition'>
  </xsl:template>

  <xsl:template match='copyright'>
    <P CLASS="COPYRIGHT">
      <xsl:element name="A">
        <xsl:attribute name="HREF">
        <xsl:value-of select="concat(generate-id(following-sibling::legalnotice[1]),'.html')"/>
        </xsl:attribute>
        Copyright
      </xsl:element>
      <xsl:text disable-output-escaping='yes'> &amp;</xsl:text>copy;
      <xsl:value-of select='year'/> by <xsl:value-of select='holder'/>
    </P>
  </xsl:template>

  <!-- return file name associated with a node -->
  <xsl:template name='getFileName'>
  	<xsl:param name='n'/>
  	<xsl:choose>
        		<xsl:when test="name($n)='chapter'">
        			<xsl:value-of select='$n/@id'/>
        	  </xsl:when>
        	
					<!-- process first level children of chapter -->
          <xsl:when test="$n/parent::chapter">
          	<!-- first section is in same file as parent chapter -->
          	<xsl:if test="count($n/preceding-sibling::section) = 0">
            <xsl:value-of select="$n/ancestor-or-self::chapter/@id"/>
          	</xsl:if>
          	<!-- second and subsequent sections are in file with name generated by generate-id() -->
          	<xsl:if test="count($n/preceding-sibling::section) > 0">
          		<xsl:value-of select="generate-id($n)"/>
          	</xsl:if>
          </xsl:when>
          
					<!-- process children of sections -->
          <xsl:when test="$n/parent::section">
          	<xsl:variable name='firstSection' select='$n/ancestor::chapter/section[1]'/>
          	<xsl:variable name='thisSection' select='$n/ancestor::section[last()]'/>
          	<xsl:choose>
          		<xsl:when test="$firstSection = $thisSection">
			            <xsl:value-of select="$n/ancestor-or-self::chapter/@id"/>
          		</xsl:when>

          		<xsl:otherwise>
          				<xsl:value-of select="generate-id($thisSection)"/>
          		</xsl:otherwise>
          	</xsl:choose>
          </xsl:when>

  	</xsl:choose>
  </xsl:template>
    
<!-- TABLE OF CONTENTS PROCESSING -->
  <xsl:template match='toc '>
    <DIV CLASS='TOC'>
      <DL>
        <DT><B><xsl:value-of select='title'/></B></DT>
        <xsl:for-each select='//chapter'>
          <xsl:call-template name='tocGen'/>
        </xsl:for-each>
      </DL>
    </DIV>	
  </xsl:template>

  <xsl:template name='tocGen'>
  	<xsl:variable name='linkdoc'>
 			<xsl:call-template name="getFileName">
 				<xsl:with-param name="n" select="." />
 			</xsl:call-template>
  	</xsl:variable>
  	  	
    <DT>
      <xsl:element name='a'>
        <xsl:attribute name='href'>
          <xsl:value-of select='concat($linkdoc, ".html#",generate-id())'/>
        </xsl:attribute>
        <xsl:value-of select="title"/>
      </xsl:element>
    </DT>

    <xsl:if test="child::section[title]">
      <DL>
        <xsl:for-each select='child::section[title]'>
            <xsl:call-template name='tocGen' />
        </xsl:for-each>
      </DL>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match='xref'>
  	<xsl:variable name='linkend' select='@linkend'/>
  	<xsl:variable name='xrefnode' select='//*[@id = $linkend]'/>
  	
 		<xsl:variable name='linkdoc'>
 			<xsl:call-template name="getFileName">
 				<xsl:with-param name="n" select="$xrefnode" />
 			</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:element name="A">
  		<xsl:attribute name="HREF">
  			<xsl:value-of select="concat($linkdoc, '.html#', @linkend)"/>
  		</xsl:attribute>

  		<!-- emit "the chapter  titled...",  "the section titled ...", "the figure titled ..." based on node type -->
  	<!--	<xsl:value-of select="concat('the ', name($xrefnode), ' titled ')"/> -->
  		<I><xsl:value-of select="$xrefnode/title"/></I>
  	</xsl:element>
  </xsl:template>
    

<!-- DEFAULT FOR UNRECOGNIZED ELEMENTS -->
  <xsl:template match='*'>
    <span style="color:red; font-weight:bold;">
      &lt;<xsl:value-of select='name()'/>&gt;
    </span>
    <xsl:apply-templates />
    <span style="color:red; font-weight:bold;">
      &lt;/<xsl:value-of select='name()'/>&gt;
    </span>
  </xsl:template>


</xsl:stylesheet>
